// $Id: Stats.cc,v 1.3.2.2 1998/09/22 07:43:43 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Stats.h"
#include <assert.h>

/* LevelMeter */

LevelMeter::LevelMeter(): theLo(0), theHi(0), theFirst(0), theLast(0), 
	theCount(0), theLevel(0) {
}

void LevelMeter::sync() {
	if (theCount++) {
		if (theLo > theLevel)
			theLo = theLevel;
		else
		if (theHi < theLevel)
			theHi = theLevel;
		theLast = theLevel;
	} else {
		theLo = theHi = theFirst = theLast = theLevel;
	}
}


/* SampleStat */

void SampleStat::reset() {
	theCount = 0;
	theSum = theSqSum = 0;
	theMax = theMin = -1;
}

void SampleStat::record(double v) {

	if (theCount++) {
		theSum += v;
		theSqSum += v*v;

		if (theMax < v)
			theMax = v;
		else
		if (theMin > v)
			theMin = v;
	} else {
		theMin = theMax = theSum = v;
		theSqSum = v*v;
	}
}

double SampleStat::var() const {

	if (theCount > 1)
		return (theSqSum - theSum*theSum/theCount) / (theCount-1);
	else
		return -1;
}

void SampleStat::report(ostream &os) const {

	os 
		<< "\t count:\t " << setw(10) << count()
		<< endl
		<< "\t min:\t "   << setw(10) << min()
		<< "\t avg:\t "   << setw(10) << avg()
		<< "\t max:\t "   << setw(10) << max()
		<< endl
		<< "\t dev:\t "   << setw(10) << stdDev()
		<< "\t rel:\t "   << setw(10) << relDev()
		<< "\t var:\t "   << setw(10) << var()
		<< endl
		<< "\t sum:\t "   << setw(10) << sum()
		<< endl;
}
