// $Id: Time.cc,v 1.8.2.1 1998/09/22 07:43:44 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Time.h"

#include <stdlib.h>
#include <iomanip.h>


Time Time::Now() {
	Time tm;
	gettimeofday(&tm, 0);
	return tm;
}

Time Time::Secd(double dsec) {
	const long sec = (long)dsec;
	return Time(sec, (long)((dsec-sec)*1e6));
}

Time &Time::operator +=(const Time &tm) {

	tv_sec += tm.tv_sec;
	tv_usec += tm.tv_usec;

	if (tv_usec >= 1000000L) {
		tv_usec -= 1000000L;
		tv_sec++;
	}

	return *this;
}

// note: negative times are confusing for humans: "-1:1" means "-0.1"
Time &Time::operator -=(const Time &tm) {

	tv_sec -= tm.tv_sec;
	tv_usec -= tm.tv_usec;

	if (tv_usec < 0) {
		tv_usec += 1000000L;
		tv_sec--;
	}

	return *this;
}

ostream &Time::print(ostream &os) const {
	if (tv_sec < 60*60*24*365 && tv_usec >= 0) {
		return printInterval(os);
	} else {
		const char osfill = os.fill();
		return os << tv_sec << '.' << setw(6) << setfill('0') << tv_usec << setfill(osfill);
	}
}

istream &Time::load(istream &is) {
	char c = '\0';
	return is >> tv_sec >> c >> tv_usec;
}

const char *Time::load(const char *buf) {
	tv_sec = strtol(buf, (char **)&buf, 10);
	if (*buf == '.')
		tv_usec = strtol(buf+1, (char **)&buf, 10);
	else
		tv_sec = -1;
	return buf;
}

ostream &Time::printInterval(ostream &os) const {

	const Time &t = *this; // historical artifact

	const int osprec = os.precision(2);
	const char osfill = os.fill();

	if (t.tv_sec < 0)
		os << t.tv_sec << '.' << setfill('0') << setw(6) << t.tv_usec << " sec";
	else
	if (t.tv_sec == 0)
		if (t.tv_usec < 1000)
			os << t.tv_usec << " usec";
		else
			os << (t.tv_usec/1000) << " msec";
	else
	if (t.tv_sec < 60)
		os << t.tv_sec << '.' << setfill('0') << setw(3) << (t.tv_usec/1000) << " sec";
	else
	if (t.tv_sec < 60*60)
		os << (t.tv_sec/60) << '.' << (10*(t.tv_sec%60)/60) << " min";
	else
		os << (t.tv_sec/(60*60)) << '.' << (10*(t.tv_sec%(60*60))/(60*60)) << " hrs";

	os.fill(osfill);
	os.precision(osprec);

	return os;
}
