#ifndef _LOADER_H_
#define _LOADER_H_ "$Id: Loader.h,v 1.3.2.1 1998/09/22 07:44:11 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AlarmClock.h"

class ostream;

// The model: Loader sends IncLoad/DecLoad events to Loadee and receives
//            feedback (current load values) from the latter
// The notion of "load" is undefined at this level, 
//            but it should be expressed as Load (currently an int)

// Load; the definition may change
typedef int Load;

// load info contains parameters for changeLoad/loadChange events
class LoadInfo {
	public:
		static LoadInfo Delta(int d) { return LoadInfo(-1, d); }
		static LoadInfo Level(int l) { return LoadInfo(l, 0); }

	public:
		LoadInfo(): theLevel(-1), theDelta(0) {}
		LoadInfo(int aLevel, int aDelta): theLevel(aLevel), theDelta(aDelta) {}

		bool valid() const { return theLevel > -1 || theDelta != 0; }

		int level() const { return theLevel; }
		int delta() const { return theDelta; }
		int curDelta(int curLevel) const { return theLevel > -1 ? (theLevel-curLevel) : theDelta; }

		ostream &print(ostream &os) const;

	protected:
		Load theLevel;  // desired or actual load level OR -1
		Load theDelta;  // desired or actual change in load level OR 0;
};

inline ostream &operator <<(ostream &os, const LoadInfo &li) { return li.print(os); }

// Loadee accepts IncLoad/DecLoad messages
class Loadee: public virtual Identity, public virtual SelfAcc {
	public:
		Loadee() {}
		virtual ~Loadee() {}

		virtual void changeLoad(const LoadInfo &) = 0;
};

// Loader emits IncLoad/DecLoad events with some pattern
class Loader: public Sleeper {
	public:
		virtual Loader *clone() const = 0;

		void loadee(Loadee *aLoadee) { theLoadee = aLoadee; }
		virtual void loadChange(const LoadInfo &) {}

		virtual void start();
		virtual void wakeUp(const Alarm &) { step(); }

	protected:
		virtual bool enough() const { return !theLoadee; }

		void step() { if (!enough()) load(); else done(); }
		virtual void load() = 0;
		virtual void warmUp() { step(); } // called from start() only
		virtual void done();

		void changeLoad(const LoadInfo &li); // sends a message

	protected:
		Loadee* theLoadee;
};

#endif
