// $Id: SafePtr.h,v 1.12.2.1 1998/09/22 07:44:24 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _SAFE_PTR_H_
#define _SAFE_PTR_H_

#include "UseAcc.h"

/*
 * XXX: The macro below ought to be a template, of course.
 * Unfortunately, g++ v2.7 and other compilers are not very good with templates
 *
 */

// SafePtr should keep thePointee and UserAcc parent
#define SafePtrDecl(Class, Pointee) \
class Pointee; \
class Class: public UserAcc {\
	public:\
		Class(): thePointee(0) {}\
		~Class() { if (safe() && theUseRec->useCount() == 1) destroyPtr(); }\
\
		operator bool() const { return safe(); }\
\
		void assign(Pointee *p) { Assert(p); reset(new UseRec()); thePointee = p; }\
		void assign(const UserAcc &ua, Pointee *p) { Assert(p); reset(ua.useRec()); thePointee = safe() ? p : 0; }\
		void assign(UseRec *ur, Pointee *p) { Assert(p); reset(ur); thePointee = safe() ? p : 0; }\
\
		Pointee &operator *() const { return *safePtr(); }\
		operator Pointee*() const { return unsafePtr(); }\
		Pointee *operator ->() const { return safePtr(); }\
\
		bool operator ==(const Class &c) const { return unsafePtr() == c.unsafePtr(); }\
		Class &operator =(int n) { Assert(!n); reset(0); return *this; }\
\
	protected:\
		Pointee *safePtr() const { Assert(safe()); return thePointee; }\
		Pointee *unsafePtr() const { return safe() ? thePointee : 0; }\
		void destroyPtr();\
\
	protected:\
		mutable Pointee *thePointee;\
}; \
inline ostream &operator <<(ostream &os, const Class &obj) { return os << (Pointee*)obj; }
// thePointee above is mutable, to avoid duplication of all "pointer" methods

#define SafePtrDefn(Class, Pointee) \
void Class::destroyPtr() { delete thePointee; }

#endif
