#ifndef _SHOOTER_H_
#define _SHOOTER_H_ "$Id: Shooter.h,v 1.5.2.1 1998/09/22 07:44:26 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "Size.h"
#include "Time.h"
#include "Array.h"

class ostream;
class istream;

// manages one object to be shot
class Shooter {
	public:
		typedef const char *Name;

	public:
		Shooter(Name aName, Size aSize, char *aBody);
		virtual ~Shooter() {}

		Name name() const { return theName; }
		Size size() const { return theSize; }

		// these should be made virtual when needed
		void shoot(char *buf);
		void restore(const char *buf);

		virtual void hton(char *buf) const = 0;
		virtual void ntoh(char *buf) const = 0;

		virtual ostream &print(ostream &os) const = 0;

	protected:
		// should be made virtual when needed
		void reset();

	protected:
		Name theName;
		Size theSize;
		char *theBody;
};

// manages all Shooters and records/saves/loads snapshots
class ShotRegistry {
	public:
		ShotRegistry();
		~ShotRegistry();

		int shotCount() const { return theShotCount; }
		Size diskSize() const { return theShotCount * theShotSize; }
		Size memSize() const { return theShots.count() * theShotSize; }
		Time startTime() const { return theFirstShot; }

		void add(Shooter *shooter);

		void startTime(Time t) { theFirstShot = theLastShot = t; }
		void preAlloc(int count); // pre allocate space for count shots
		void snapShot(Time shotTimez);

		void save(ostream &) const;        // mem to binary format
		void load(istream &) { abort(); }  // binary format to mem
		void print(ostream &) { abort(); } // mem to readable stream

		// binary format to readable stream, one shot at a time
		void print(istream &, ostream &);

	protected:
		void saveHeader(ostream &) const;
		int loadHeader(istream &); // returns number of shots to expect
		void printHeader(ostream &) const;

		void addShotBuf(); // adds one more buffer for shots

	protected:
		Array<Shooter*> theShooters;
		Array<char*> theShots;

		Size theShotSize;
		Time theFirstShot; // the start of the first shot
		Time theLastShot;  // the end of the last shot
		int theShotCount;  // number of shots actually made
};

// this can be moved as a parameter of Shooter's constructor if more than one needed
extern ShotRegistry *TheShotRegistry;


// internal object to initialize ShotRegistry, ignore
class ShotRegistryLibCounter {
	public:
		ShotRegistryLibCounter();
		~ShotRegistryLibCounter();

	protected:
		static int theUseCount;
};
static ShotRegistryLibCounter TheShotRegistryLibCounter;

#endif
