// $Id: XactThread.h,v 1.9.2.1 1998/09/22 07:44:35 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _XACT_THREAD_H_
#define _XACT_THREAD_H_

#include "Thread.h"
#include "ConnMutex.h"

class XactRec;

SafePtrDecl(XactThreadPtr, XactThread);

// common class for all transaction threads
// note: xaction threads are usually run under PushPull control
class XactThread: public Thread {
	public:
		XactThread(): theRec(0) {}

		void conn(const ConnMx &aConn) { theConn = aConn; }
		void partner(XactThread *aPartner) { thePartner.assign(aPartner->useRec(), aPartner); }
		void record(XactRec *aRec) { theRec = aRec; }
		
		// ``rpc'' handlers
		virtual void bufChanged() { beActive(thRunnable); }
		void setLocalLimit(Size aLimit) { Assert(theStartOffset >= 0); doSetMaxOffset(theStartOffset+aLimit); beActive(thRunnable); }
		void setMaxOffset(Size anOffset) { doSetMaxOffset(anOffset); beActive(thRunnable); }

		virtual void checkProgress() { if (enough()) { state(thDone); printState("done"); } }
		virtual void init() { theStartOffset = workOffset(); }

	protected:
		bool overLimit() const { return workOffset() >= theMaxOffset && theMaxOffset >= 0; }
		virtual bool enough() const { return !theConn || !theConn->good() || overLimit() || workBuf().exhausted(); }

		virtual Size workOffset() const = 0;
		virtual IOBuffer &workBuf() const = 0;

		// be sure start offset is set before calling these
		Size sizeTotal() const { return theMaxOffset - theStartOffset; }
		Size sizeMade() const { return workOffset() - theStartOffset; }
		Size sizeLeft() const { return theMaxOffset - workOffset(); }
		void maxOffsetHere() { maxOffset(workOffset()); }

		void localLimit(Size aLimit) { Assert(theStartOffset >= 0); maxOffset(theStartOffset + aLimit); }
		void maxOffset(Size anOffset) { doSetMaxOffset(anOffset); sendMaxOffsetChange(theMaxOffset); }

		void sendBufChange() { thSignal(thePartner, bufChanged); }
		void sendLocalLimitChange(Size limit) { thCall(thePartner, setLocalLimit, limit); }
		void sendMaxOffsetChange(Size offset) { thCall(thePartner, setMaxOffset, offset); }

		void printState(const char *label); // debugging

	private:
		// used internally by maxOffset and setMaxOffset
		void doSetMaxOffset(Size anOffset) { Assert(anOffset >= theStartOffset && anOffset >= theMaxOffset); theMaxOffset = anOffset; }

	protected:
		ConnMx theConn;
		XactThreadPtr thePartner;
		Size theStartOffset; // "work buffer" offset when we started
		Size theMaxOffset;   // stop when "work buffer" is over this offset
		XactRec *theRec;
};

#endif
