#!/usr/bin/perl -w

#
# produces ps-like output based on polygraph's log
#

# 901250203.686502# Thread.cc:16: thread PushPull 10:16:21 started
# started++
#
# 901250203.686502# PushPull.cc:17:       10:16:21: asked lock for 00:19 and 00:18
# waiting++
#
# 901250203.686502# PushPull.cc:7:        10:16:21: got lock for 00:19 and 00:18
# waiting--
# running++
#
# 901250203.757605# Thread.cc:30: thread PushPull 10:16:22 stopped
# finished++

my %state = (
	started => 1,
	waiting => 2,
	running => 4,
	finished => 8
);

my %KnownThreads = ();

my ($startCount, $waitCount, $runCount, $finishCount) = (0) x 4;

&reportHeader();

while(<>) {
	next unless /PushPull/;
	my ($time) = (/(\d+\.\d+)/);
	my $change = '?';
	if (/([\d:]+)\s*started/) {
		my $id = $1;
		die("$time: twins") if defined $KnownThreads{$id};
		$KnownThreads{$id} = $state{started};
		$startCount++;
		$change = 'S';
	}
	elsif (/([\d:]+):\s*asked lock/) {
		my $id = $1;
		my $th = $KnownThreads{$id};
		die("$time: unknonwn") unless defined $th;
		die("$time wrong state") unless $th == $state{started};
		$KnownThreads{$id} = $state{waiting};
		$waitCount++;
		$change = 'W';
	}
	elsif (/([\d:]+):\s*got \s*lock/) {
		my $id = $1;
		my $th = $KnownThreads{$id};
		die("$time: unknonwn") unless defined $th;
		die("$time wrong state") unless $th == $state{waiting};
		$waitCount--;
		$KnownThreads{$id} = $state{running};
		$runCount++;
		$change = 'R';
	}
	elsif (/([\d:]+)\s*stopped/) {
		my $id = $1;
		my $th = $KnownThreads{$id};
		die("$time: unknonwn") unless defined $th;
		die("$time wrong state") unless $th == $state{running};
		$runCount--;
		undef $KnownThreads{$id};
		$finishCount++;
		$change = 'T';
	} else {
		next;
	}
	&report($time, $change);
}

sub report {
	my ($prefix, $change) = @_;
	printf("%17s  %5d %5d %5d %5d %5d  %s\n",
		$prefix, $startCount, $waitCount, $runCount, $finishCount,
		$startCount - $finishCount,
		$change);
}

sub reportHeader {
	printf("\#%16s  %5s %5s %5s %5s %5s %5s %s\n",
		'time', 'start', 'wait', 'run', 'finish', 'level', 'change');
}

